// MSPECS API methods
// Written by Sergio Crisóstomo, based on the work by Gustaf Naeser.

const fs = require('fs');
const https = require('https');
const querystring = require('querystring');
const credentials = {};

function options(url = '', method, data) {
	const auth = Object.keys(credentials).map(k => credentials[k]).join(':');

	var opts = {
		host: 'api.mspecs.se',
		path: url,
		method: method,
		rejectUnauthorized: false,
		headers: {
			'Authorization': 'Basic ' + new Buffer(auth).toString('base64')
		}
	};

	if(data) { // if there is data we need to add some headers
		opts.headers['Content-Type'] = 'application/x-www-form-urlencoded';
		opts.headers['Content-Length'] = data.length;
	}

	return opts;
}

/*
	Unified way of making requests.

	Note that we expect that we get json back and have minimal error
	handling.

	Args:
	`url` is the target
	`dataObj` is used in POST and PUT
	`meth` is GET | POST | PUT

	Returns:
	cb(error, result)
*/
function requestData(url, dataObj, meth, cb) {
	return new Promise((resolve, reject) => {

		if (!credentials.username || !credentials.password) return reject(`
			The username and password are not set.
			Please set them using setCredentials(username, password);
		`);

		const data = querystring.stringify(dataObj)
		const req = https.request(options(url, meth, data), function(res) {
			let body = "";
			res.setEncoding('utf8');
			res.on('data', chunk => body += chunk);
			res.on('end', () => {
				var sc = res.statusCode;
				if (sc == 200 || sc == 201) resolve(JSON.parse(body)); // OK!
				else reject(body); // KO!
			});
		});
		req.on('error', reject);
		if (dataObj !== '') req.write(data);
		req.end();
	}).catch(err => console.log(err));
}

module.exports = {
	getData: (url, data) => {
		if (typeof data == 'string') (url = `${url}/${data}`, data = null);
		return requestData(url, data || null, 'GET');
	},
	getList: (url, cb) => requestData(url, null, 'GET'),
	putData: (url, data) => requestData(url, data, 'PUT'),
	postData: (url, data) => requestData(url, data, 'POST'),

	setCredentials: (username, password) => (
		credentials.username = username, credentials.password = password
	)
}
